/******************************************************************/
/* igluTexture2D.cpp                                              */
/* -----------------------                                        */
/*                                                                */
/* The file defines an image class that stores a 2D texture.      */
/*     This is very similar to the image class, but also defines  */
/*     access patterns for the texture with interpolation.        */
/*                                                                */
/* Chris Wyman (10/26/2006)                                       */
/******************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <GL/glew.h>
#include "iglu.h"

using namespace iglu;


IGLUPixelBufferTexture2D::IGLUPixelBufferTexture2D( GLenum format, int width, int height, 
							                        unsigned int flags, bool initializeImmediately ) :
	IGLUTexture(), m_width( width ), m_height( height )
{
	SetTextureParameters( flags );

	m_pixelFormat   = 0;

	if (initializeImmediately) Initialize();
}


IGLUPixelBufferTexture2D::~IGLUPixelBufferTexture2D()
{
	if (m_filename) free( m_filename );
}

void IGLUPixelBufferTexture2D::UpdateTextureParameters( void )
{
	glBindTexture( GL_TEXTURE_2D, m_texID );
	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, m_minFilter );
	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, m_magFilter );
	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, m_sWrap );
	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, m_tWrap );
	glBindTexture( GL_TEXTURE_2D, 0 );
}

void IGLUPixelBufferTexture2D::SetTextureParameters( unsigned int flags )
{
	m_minFilter     = GetMinification( flags );
	m_magFilter     = GetMagnification( flags );
	m_sWrap         = GetSWrap( flags );
	m_tWrap         = GetTWrap( flags );
	m_mipmapsNeeded = UsingMipmaps( flags );

	if (m_initialized)
		UpdateTextureParameters();
}

void IGLUPixelBufferTexture2D::Initialize( void )
{
	// Make sure to avoid doubly-calling initialize
	if (m_initialized) return;

	// Double check that we *can* initialize.
	CheckValidOpenGL( __FILE__, __FUNCTION__, __LINE__ );

	// OK, now we can set up our texture
	UpdateTextureParameters();

	glBindTexture( GL_TEXTURE_2D, m_texID );
	GLenum format = 0;
	glTexImage2D( GL_TEXTURE_2D, 0, format,  
						   m_width, m_height, 0,
						   GetBaseInternalFormat(format), 0, //m_texImg->GetGLDatatype(), 
						   0 );
	if (m_mipmapsNeeded)
		glGenerateMipmap( GL_TEXTURE_2D );

	glBindTexture( GL_TEXTURE_2D, 0 );

	// Done initializing
	m_initialized = true;
}

