/******************************************************************/
/* igluFLTKGLWindow.cpp                                           */
/* -----------------------                                        */
/*                                                                */
/* An ugly little file that overrides the fragile OpenGL context  */
/*    creation code inside the Fl_Gl_Window class.
/*                                                                */
/*                                                                */
/* Chris Wyman (02/21/2012)                                       */
/******************************************************************/


// Needed to #include correct stuff from <FL/x.H>
#define FL_INTERNALS

#include <GL/glew.h>
#include <GL/wglew.h>
#  include <FL/Fl.H>
#  include <FL/x.H>
#  include <FL/gl_draw.H>
#  include <FL/fl_utf8.h>

#include "iglu.h"
#include "iglu/window/igluFLTKGLWindow.h"
#include <cassert>

using namespace iglu;

namespace {  // Anonymous namespace.  Avoid clashing with FLTK.

// FLTK's Fl_Gl_Choice structure.  Renamed for easy access inside this file.
//    This is redefined, since it's not in public FLTK headers.
class My_Gl_Choice {
  int mode;
  const int *alist;
  My_Gl_Choice *next;
public:
#ifdef WIN32
  int pixelformat;	
  PIXELFORMATDESCRIPTOR pfd;
#elif defined(__APPLE_QUARTZ__)
  AGLPixelFormat pixelformat;
#else
  XVisualInfo *vis;	
  Colormap colormap;	
#endif
  static My_Gl_Choice *find(int mode, const int *);
};

My_Gl_Choice *first = 0;

#if defined(WIN32)

// This method takes the Window DC, creates a context, uses that context to load
//   the wglCreateContextAttribsARB() method from the vendor's OpenGL driver DLL,
//   then kills off the temporary context we created.  
// This seems *real* dumb to have to create a GL context to load the function needed
//   to create our real GL context.  But that's what it looks like is needed...
// Note: Windows specific.
PFNWGLCREATECONTEXTATTRIBSARBPROC ctxFunc=0;
void InitWGLFuncPtrs(HDC hDC)
{
    HGLRC hTempRC = wglCreateContext(hDC);
	HGLRC oldRC   = wglGetCurrentContext();
    wglMakeCurrent(hDC, hTempRC);
    ctxFunc = (PFNWGLCREATECONTEXTATTRIBSARBPROC) wglGetProcAddress("wglCreateContextAttribsARB");
    wglMakeCurrent(hDC, oldRC);
    wglDeleteContext(hTempRC);
}

// Windows specific context creation.  This is only slightly modified from FLRK's function
//   by the same name.  The modifications are:  1) remove colormap crap, as I don't plan to
//   ever used colormapped OpenGL again.  2)  Uses wglCreateContextAttribsARB() whenever it
//   can be loaded from the OpenGL driver.
GLContext fl_create_gl_context(Fl_Window* window, const My_Gl_Choice* g) 
{
	Fl_X* i = Fl_X::i(window);
	HDC hdc = i->private_dc;
	if (!hdc) {
		hdc = i->private_dc = GetDCEx(i->xid, 0, DCX_CACHE);
		fl_save_dc(i->xid, hdc);
		SetPixelFormat(hdc, g->pixelformat, (PIXELFORMATDESCRIPTOR*)(&g->pfd));
	}

	// Grab IGLU's desired OpenGL context parameters
	int *attriblist = IGLUContext::GetContextAttribList();

	// Load the method wglCreateContextAttribsARB() into variable ctxFunc.
	InitWGLFuncPtrs(hdc);

	// Create our context
	GLContext context = (ctxFunc) ? ctxFunc(hdc, 0, attriblist) : wglCreateContext(hdc);	  
	return context;
}

} // end anonymous namespace

#else
GLContext fl_create_gl_context(Fl_Window* window, const My_Gl_Choice* g) 
{
	printf( "IGLU's Fl_BetterGl_Window class currently only works under windows!\n");
	assert(false);

	// If you need to do this for Linux / Mac, checkout the code in Fl_Gl_Choice.cxx
	//    in the FLTK distribution (part of fltkgl.lib/fltkgl.a/fltkgl.so).  This code
	//    copied here was from FLTK 1.3.2.
	// You'll see a number of methods 'fl_create_gl_context()'  
	// The Linux one should be easy to port, it's about 8 lines long, and you just need
	//    to change the line involving glXCreateContext() to first load glXCreateContextAttribARB(),
	//    then use a ? : operator (as above) to pick betweent the loaded function and the glXCreateContext().
	// Loading the glXCreateContextAttribARB() should be virtually identical to InitWGLFuncPtrs(), only
	//    you'll use glX*() functions rather than wgl*() methods.
}
#endif



void Fl_BetterGl_Window::make_current( void )
{
	// Before calling make_current(), check if the window already has a context.
	//   If not, go ahead and create a context with the appropriate OpenGL profile,
	//   then stuff it into the Fl_Gl_Window, as if it had created it itself.
	if (!context())
	{
		int mode = int(Fl_Gl_Window::mode());
		My_Gl_Choice *g = My_Gl_Choice::find( mode, 0 );
		GLContext ctx = fl_create_gl_context(this, g);
		Fl_Gl_Window::context( ctx, 1 );
	}

	// Ok, we've created a context if needed.  Go and make it current!
	Fl_Gl_Window::make_current();
}







// this assumes one of the two arguments is zero:
// We keep the list system in Win32 to stay compatible and interpret
// the list later...
My_Gl_Choice *My_Gl_Choice::find(int m, const int *alistp) {
  My_Gl_Choice *g;
  
  for (g = first; g; g = g->next)
    if (g->mode == m && g->alist == alistp) 
      return g;

#if defined(USE_X11)    
  const int *blist;
  int list[32];
    
  if (alistp)
    blist = alistp;
  else {
    int n = 0;
    if (m & FL_INDEX) {
      list[n++] = GLX_BUFFER_SIZE;
      list[n++] = 8; // glut tries many sizes, but this should work...
    } else {
      list[n++] = GLX_RGBA;
      list[n++] = GLX_GREEN_SIZE;
      list[n++] = (m & FL_RGB8) ? 8 : 1;
      if (m & FL_ALPHA) {
	list[n++] = GLX_ALPHA_SIZE;
	list[n++] = (m & FL_RGB8) ? 8 : 1;
      }
      if (m & FL_ACCUM) {
	list[n++] = GLX_ACCUM_GREEN_SIZE;
	list[n++] = 1;
	if (m & FL_ALPHA) {
	  list[n++] = GLX_ACCUM_ALPHA_SIZE;
	  list[n++] = 1;
	}
      }
    }
    if (m & FL_DOUBLE) {
      list[n++] = GLX_DOUBLEBUFFER;
    }
    if (m & FL_DEPTH) {
      list[n++] = GLX_DEPTH_SIZE; list[n++] = 1;
    }
    if (m & FL_STENCIL) {
      list[n++] = GLX_STENCIL_SIZE; list[n++] = 1;
    }
    if (m & FL_STEREO) {
      list[n++] = GLX_STEREO;
    }
#    if defined(GLX_VERSION_1_1) && defined(GLX_SGIS_multisample)
    if (m & FL_MULTISAMPLE) {
      list[n++] = GLX_SAMPLES_SGIS;
      list[n++] = 4; // value Glut uses
    }
#    endif
    list[n] = 0;
    blist = list;
  }
    
  fl_open_display();
  XVisualInfo *visp = glXChooseVisual(fl_display, fl_screen, (int *)blist);
  if (!visp) {
#    if defined(GLX_VERSION_1_1) && defined(GLX_SGIS_multisample)
    if (m&FL_MULTISAMPLE) return find(m&~FL_MULTISAMPLE,0);
#    endif
    return 0;
  }

#elif defined(__APPLE_QUARTZ__)
  // warning: the Quartz version should probably use Core GL (CGL) instead of AGL
  const int *blist;
  int list[32];
   
  if (alistp)
    blist = alistp;
  else {
    int n = 0;
    if (m & FL_INDEX) {
      list[n++] = AGL_BUFFER_SIZE;
      list[n++] = 8; // glut tries many sizes, but this should work...
    } else {
      list[n++] = AGL_RGBA;
      list[n++] = AGL_GREEN_SIZE;
      list[n++] = (m & FL_RGB8) ? 8 : 1;
      if (m & FL_ALPHA) {
        list[n++] = AGL_ALPHA_SIZE;
        list[n++] = (m & FL_RGB8) ? 8 : 1;
      }
      if (m & FL_ACCUM) {
        list[n++] = AGL_ACCUM_GREEN_SIZE;
        list[n++] = 1;
        if (m & FL_ALPHA) {
          list[n++] = AGL_ACCUM_ALPHA_SIZE;
          list[n++] = 1;
        }
      }
    }
    if (m & FL_DOUBLE) {
      list[n++] = AGL_DOUBLEBUFFER;
    }
    if (m & FL_DEPTH) {
      list[n++] = AGL_DEPTH_SIZE; list[n++] = 24;
    }
    if (m & FL_STENCIL) {
      list[n++] = AGL_STENCIL_SIZE; list[n++] = 1;
    }
#    ifdef AGL_STEREO
    if (m & FL_STEREO) {
      list[n++] = AGL_STEREO;
    }
#    endif
    list[n] = AGL_NONE;
    blist = list;
  }
  fl_open_display();
  AGLPixelFormat fmt = aglChoosePixelFormat(NULL, 0, (GLint*)blist);
  if (!fmt) return 0;
  
#elif defined(WIN32)

  // Replacement for ChoosePixelFormat() that finds one with an overlay
  // if possible:
  if (!fl_gc) fl_GetDC(0);
  int pixelformat = 0;
  PIXELFORMATDESCRIPTOR chosen_pfd;
  for (int i = 1; ; i++) {
    PIXELFORMATDESCRIPTOR pfd;
    if (!DescribePixelFormat(fl_gc, i, sizeof(pfd), &pfd)) break;
    // continue if it does not satisfy our requirements:
    if (~pfd.dwFlags & (PFD_DRAW_TO_WINDOW | PFD_SUPPORT_OPENGL)) continue;
    if (pfd.iPixelType != ((m&FL_INDEX)?PFD_TYPE_COLORINDEX:PFD_TYPE_RGBA)) continue;
    if ((m & FL_ALPHA) && !pfd.cAlphaBits) continue;
    if ((m & FL_ACCUM) && !pfd.cAccumBits) continue;
    if ((!(m & FL_DOUBLE)) != (!(pfd.dwFlags & PFD_DOUBLEBUFFER))) continue;
    if ((!(m & FL_STEREO)) != (!(pfd.dwFlags & PFD_STEREO))) continue;
    if ((m & FL_DEPTH) && !pfd.cDepthBits) continue;
    if ((m & FL_STENCIL) && !pfd.cStencilBits) continue;
    // see if better than the one we have already:
    if (pixelformat) {
      // offering non-generic rendering is better (read: hardware accelleration)
      if (!(chosen_pfd.dwFlags & PFD_GENERIC_FORMAT) &&
          (pfd.dwFlags & PFD_GENERIC_FORMAT)) continue;
      // offering overlay is better:
      else if (!(chosen_pfd.bReserved & 15) && (pfd.bReserved & 15)) {}
      // otherwise more bit planes is better:
      else if (chosen_pfd.cColorBits > pfd.cColorBits) continue;
      else if (chosen_pfd.cDepthBits > pfd.cDepthBits) continue;
    }
    pixelformat = i;
    chosen_pfd = pfd;
  }
  //printf("Chosen pixel format is %d\n", pixelformat);
  if (!pixelformat) return 0;
#else
# error platform unsupported
#endif

  g = new My_Gl_Choice;
  g->mode = m;
  g->alist = alistp;
  g->next = first;
  first = g;

#if defined(USE_X11)
  g->vis = visp;

  if (/*MaxCmapsOfScreen(ScreenOfDisplay(fl_display,fl_screen))==1 && */
      visp->visualid == fl_visual->visualid &&
      !fl_getenv("MESA_PRIVATE_CMAP"))
    g->colormap = fl_colormap;
  else
    g->colormap = XCreateColormap(fl_display, RootWindow(fl_display,fl_screen),
				  visp->visual, AllocNone);
#  elif defined(WIN32)
  g->pixelformat = pixelformat;
  g->pfd = chosen_pfd;
#  elif defined(__APPLE_QUARTZ__)
  // warning: the Quartz version should probably use Core GL (CGL) instead of AGL
  g->pixelformat = fmt;
#  else
#    error unsupported platform
#  endif

  return g;
}